﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:02:20 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("wowevents.lua")

-------------------------------------------------------------------------------
-- WOW GAME EVENT HANDLERS
-------------------------------------------------------------------------------


function SpeakinSpell:RegisterAllEvents()
	-- register for spellcasting events
	-- which is our hook too know when to speak for a spell
	-- among other things
	
	-- startup and loading events
	self:RegisterEvent("VARIABLES_LOADED")
	
	-- combat events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

	-- enter and exit combat
	-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
	--		Those 2 events are auto-attack on/off notifications
	--		To detect when the combat flag goes on and off from getting/losing aggro
	--		which is what we generally mean by "entering combat" and "exiting combat"
	--		we must check for Regen enabled/disabled, which is our only valid notification
	--		see also: http://www.wowwiki.com/Events/Combat
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	-- change zones --
	--self:RegisterEvent("MINIMAP_ZONE_CHANGED") --for subzone changes - does not appear to be needed
	self:RegisterEvent("ZONE_CHANGED_INDOORS") --for subzone changes inside an instance
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA") --for large scale zone changes
	self:RegisterEvent("ZONE_CHANGED") --covers both small and large scale zone changes?
end


---------------------------------------------------------------------------
-- STARTUP AND LOADING EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:VARIABLES_LOADED()
	self:OnVariablesLoaded() -- redirected to be near OnInitialize
end



---------------------------------------------------------------------------
-- COMBAT EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, auraType)
	local funcname = "SPELL_AURA_APPLIED"
	-- we're only concerned with buffs we receive
	if dstName ~= UnitName("player") then
		--self:DebugMsg(funcname, "not cast on you")
		return
	end
	
	self:DebugMsg(funcname, "( srcName, dstName, auraType, spellName )")
	self:DebugMsgDumpString("srcName",srcName)
	self:DebugMsgDumpString("dstName",dstName)
	self:DebugMsgDumpString("auraType",auraType)
	self:DebugMsgDumpString("spellName",spellName)
	
	local DetectedEventStub = {
		-- event descriptors
		name = spellName,
		type = "SPELL_AURA_APPLIED_"..auraType,
		--rank = "", -- no rank known
		-- event-specific data for substitutions
		caster = srcName,
		target = dstName,
		spellid = spellId,
	}
	
	-- buffs cast on me by someone other than me are treated differently than buffs I cast on myself
	if srcName ~= UnitName("player") then
		self:DebugMsg(funcname, "not cast by you")
		DetectedEventStub.type = DetectedEventStub.type.."_FOREIGN"
	end
	
	-- process the spell
	-- this function is shared with the UNIT_SPELLCAST_SENT event handler
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:COMBAT_LOG_EVENT_UNFILTERED(_,timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, ...)
	--self:DebugMsg("COMBAT_LOG_EVENT_UNFILTERED",tostring(eventtype))
	if ((eventtype == "SPELL_AURA_APPLIED") or
		(eventtype == "SPELL_AURA_REFRESH")) then -- report rebuffs the same way
		self:SPELL_AURA_APPLIED(timestamp, "SPELL_AURA_APPLIED", srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, ...)
	end
end


function SpeakinSpell:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
	local funcname = event

	self:DebugMsg(funcname,"(caster, spellname, spellrank, target)")
	--self:DebugMsgDumpString("event",event) -- the event name is already the funcname
	self:DebugMsgDumpString("caster",caster)
	self:DebugMsgDumpString("spellname",spellname)
	self:DebugMsgDumpString("spellrank",spellrank)
	self:DebugMsgDumpString("target",self.RuntimeData.LastSpellcastSentTarget)
	
	-- only speak for spells cast by the player, not anyone else
	if caster == "player" then
		caster = UnitName("player")
	else
		self:DebugMsg(funcname,"caster is not player")
		return
	end
	
	-- process the spell
	local DetectedEventStub = {
		-- event descriptors
		type = event,
		name = spellname,
		rank = spellrank,
		-- event-specific data for substitutions
		caster = caster,
		target = self.RuntimeData.LastSpellcastSentTarget,
		spellid = self:GetSpellID(spellname, spellrank),
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:UNIT_SPELLCAST_SENT(event, caster, spellname, spellrank, target)
	self.RuntimeData.LastSpellcastSentTarget = target --subsequent UNIT_SPELLCAST notifications don't include this target info, but it applies
	self.RuntimeData.LastInterrupt = nil -- new spell cast can be interrupted again - see UNIT_SPELLCAST_INTERRUPTED
	self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_INTERRUPTED(event, caster, spellname, spellrank)
	-- these messages can be redundant for a single interrupt notified 3 times
	if self.RuntimeData.LastInterrupt == spellname then
		self:DebugMsg(nil,"suppressed redundant UNIT_SPELLCAST_INTERRUPTED for "..spellname)
		return
	end
	self.RuntimeData.LastInterrupt = spellname
	self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_FAILED(event, caster, spellname, spellrank)
	-- we don't get a UNIT_SPELLCAST_SENT or any target indication for fails
	self.RuntimeData.LastSpellcastSentTarget = nil
	self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end


function SpeakinSpell:UNIT_SPELLCAST_CHANNEL_START  (event, caster, spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_CHANNEL_STOP   (event, caster, spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_STOP           (event, caster, spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_SUCCEEDED      (event, caster, spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, spellname, spellrank)
end


---------------------------------------------------------------------------
-- ENTERING AND EXITING COMBAT
---------------------------------------------------------------------------
-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
--		Those 2 events are auto-attack on/off notifications
--		To detect when the combat flag goes on and off from getting/losing aggro
--		which is what we generally mean by "entering combat" and "exiting combat"
--		we must check for Regen enabled/disabled, which is our only valid notification
--		see also: http://www.wowwiki.com/Events/Combat


function SpeakinSpell:PLAYER_REGEN_DISABLED()
	local funcname = "PLAYER_REGEN_DISABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = true
	self:ResetOncePerCombatFlags()

	-- Signal the Entering Combat event
	self:DebugMsg(funcname, "Signal System Event: Entering Combat")
	local DetectedEventStub = {
		-- event descriptors
		name = L.EVENTNAMES["Entering Combat"],
		type = "EVENT",
		-- event-specific data for substitutions
		-- None
	}
	self:OnSpeechEvent( DetectedEventStub ) 
end


function SpeakinSpell:PLAYER_REGEN_ENABLED()
	local funcname = "PLAYER_REGEN_ENABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = false
	self:ResetOncePerCombatFlags()
	
	-- Signal the Exiting Combat event
	self:DebugMsg(funcname, "Signal System Event: Exiting Combat")
	local DetectedEventStub = {
		-- event descriptors
		name = L.EVENTNAMES["Exiting Combat"],
		type = "EVENT",
		-- event-specific data for substitutions
		-- None
	}
	self:OnSpeechEvent( DetectedEventStub ) 
end


---------------------------------------------------------------------------
-- MOVING TO A DIFFERENT ZONE OR REGION
---------------------------------------------------------------------------


function SpeakinSpell:OnZoneChange(minoronly)
	local funcname = "OnZoneChange"

	-- Signal the zone change event
	self:DebugMsg(funcname, "Signal System Event: Zone Changed, minoronly="..tostring(minoronly))
	local DetectedEventStub = {
		-- event descriptors
		name = L.EVENTNAMES["Zone Changed"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on himself
		caster = UnitName("player"),
		target = UnitName("player"),
		-- event-specific data for substitutions
		-- None
	}
	
	if minoronly then
		DetectedEventStub.name = L.EVENTNAMES["Subzone Changed"]
	end
	
	self:OnSpeechEvent( DetectedEventStub ) 
end


--for subzone changes -- does not appear to be needed
--[[
function SpeakinSpell:MINIMAP_ZONE_CHANGED()
	local funcname = "MINIMAP_ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end
--]]

--for subzone changes inside an instance
function SpeakinSpell:ZONE_CHANGED_INDOORS()
	local funcname = "ZONE_CHANGED_INDOORS"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end

--for large scale zone changes
function SpeakinSpell:ZONE_CHANGED_NEW_AREA()
	local funcname = "ZONE_CHANGED_NEW_AREA"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(false)
end

--covers both small and large scale zone changes?
function SpeakinSpell:ZONE_CHANGED()
	local funcname = "ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end
